/*
 * clock for root window.
 */

/*
 * - finish alarms set up
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <locale.h>
#include <string.h>
#include <strings.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include "config.h"
#include "options.h"
#include "rootapp.h"

#define EVENTMASK VisibilityChangeMask | ExposureMask | PropertyChangeMask

/* Global varriables */
static int end_program = FALSE;

static char  datastr[MAXSTRLEN + 1];
static char *fontname   = NULL;
static char *geometry   = NULL;
static char *colorname  = NULL;
static char *format     = NULL;


/* Command line options and config file entries */
static Option opttab[] = {
    {'h', NULL,      NULL, O_HELP,   NULL,       FALSE, "display help"},
    {'v', NULL,      NULL, O_VERS,   NULL,       FALSE, "display version"},
    {'c', "color",   NULL, C_STRING, &colorname, FALSE, "text color"},
    {'f', "font",    NULL, C_STRING, &fontname,  FALSE, "font to use"},
    {'g', "geom",    NULL, C_STRING, &geometry,  FALSE, "set geometry"},
    {'t', "text",    NULL, C_STRING, &format,    FALSE, "text to display (%02d for workspace #)"},
    {0,   NULL,      NULL, C_NONE,   NULL,       FALSE, NULL}
};



/****** UPDATING FUNCTIONS ******/

void get_workspace_num() {
    int actual_format, screen = DefaultScreen(display);
    int *ws_num, num;
    unsigned long n, n2;
    Atom actual_atom;
    static Atom ws_atom = None;
    Window root = RootWindow(display, screen);

    if (ws_atom == None) {
        if ((ws_atom = XInternAtom(display, "_PWM_WORKSPACE_NUM", True)) == None) {
            fprintf(stderr, "warning: can't get atom _PWM_WORKSPACE_NUM\n");
	}
    }

    if (XGetWindowProperty(display, root, ws_atom, 0L, 1L, False, XA_INTEGER,
		&actual_atom, &actual_format, &n, &n2, 
		(unsigned char **)&ws_num) != Success) {
	fprintf(stderr, "warning: can't get property\n");
    }

    if (ws_num) num = *ws_num + 1;
    else        num = 1;

    if (format)
        snprintf(datastr, MAXSTRLEN, format, num);
    else
        snprintf(datastr, MAXSTRLEN, "workspace %02d", num);
    FREE(ws_num);
}



/****** PROCESSING FUNCTIONS ******/

void main_loop(RootApp *app) {
    XEvent event;
    int cont = TRUE;

    if (colorname) set_color(colorname);
    get_workspace_num();
    root_print(app, datastr, 1, 1);
    display_app(app);
    while (cont) {
        nextevent_or_timeout(&event, 0);
        get_workspace_num();
        clear_bg(app);
        root_print(app, datastr, 1, 1);
        display_app(app);
        if (end_program) cont = FALSE;
    }
}


/****** SIGNALS ******/

void install_handler();


void signals_handler(int sig) {
    switch (sig) {
        case SIGINT:
        case SIGTERM:
        case SIGQUIT:
            end_program = TRUE;
            break;
    }
}


void install_handler() {
    signal(SIGUSR1, signals_handler);
    signal(SIGTERM, signals_handler);
    signal(SIGQUIT, signals_handler);
    signal(SIGINT,  signals_handler);
}


void manage_signals() {
    struct sigaction sa;

    sa.sa_handler = SIG_IGN;
#ifdef SA_NOCLDWAIT
    sa.sa_flags = SA_NOCLDWAIT;
#else
    sa.sa_flags = 0;
#endif
    sigemptyset(&sa.sa_mask);
    sigaction(SIGCHLD, &sa, NULL);
    install_handler();
}


/****** MAIN ******/


int main(int argc, char *argv[]) {
    RootApp *app;

    manage_signals();
    setlocale(LC_ALL, "");
    parse_argv(opttab, argc, argv);
    if (format)
        snprintf(datastr, MAXSTRLEN, format, 0);
    else
        sprintf(datastr, "workspace 01");
    app = create_app(EVENTMASK, geometry, fontname, datastr);
    main_loop(app);
    close_app(app);
    free_vars(opttab);

    return 0;
}
